<?php 

class StringBuilder {

	private $str = array();

	public function __construct() { }

	public function append($str) {
		$this->str[] = $str;
	}

	public function toString() {
		return implode($this->str);
	}
}

class Markdown {
	const 	BLANK = 0;
	const	PARA  = 1;
	const	LST   = 2;
	const	QUOTE = 3;

	public $this;
	public $out;
	public $input;

	public function setState($s)
	{
		if($s == $this->state)
			return;
		switch($this->state) {
		case PARA:
			$this->out->append("</p>\n");
			break;
		case LST:
			$this->out->append("</ul>\n");
			break;
		case QUOTE:
			$this->out->append("</blockquote>\n");
			break;
		}

		$this->state = $s;

		switch($this->state) {
		case PARA:
			$this->out->append("<p>");
			break;
		case LST:
			$this->out->append("<ul>\n");
			break;
		case QUOTE:
			$this->out->append("<blockquote>\n");
			break;
		}
	}

	public function __construct()
	{
	}

	public function convert($in)
	{
		$this->input = $in;
		$len = strlen($this->input);
		$pos = 0;
		$this->out = new StringBuilder();
		$this->state = BLANK;
		while($pos < $len) {
			$line_end = strpos($this->input,"\n",$pos);
			if($line_end === false) {
				$line_end = $len;
			}
			else {
				$line_end++;
			}
			if($this->input[$pos]=='#') {
				$this->setState(BLANK);
				$this->out->append("<h1>");
				$this->processText($pos+1,$line_end);
				$this->out->append("</h1>\n");
			}
			else if($this->input[$pos]=='-') {
				$this->setState(LST);
				$this->out->append("<li>");
				$this->processText($pos+1,$line_end);
				$this->out->append("</li>\n");
			}
			else if($this->input[$pos]=='>') {
				$this->setState(QUOTE);
				$this->processText($pos+1,$line_end);
				$this->out->append("\n");
			}
			else {
				$line = substr($this->input,$pos,$line_end-$pos);
				$is_blank = strlen(trim($line))==0;
				if($is_blank)
					$this->setState(BLANK);
				else { 
					$this->setState(PARA);
					$this->processText($pos,$line_end);
					$this->out->append("\n");
				}
			}
			$pos = $line_end;
		}
		
		$this->setState(BLANK);

		return $this->out->toString();
	}

	public function processText($start,$end)
	{
		$line = substr($this->input,$start,$end-$start);
		$line = trim($line);
		$key = array("<" , ">", "&" , "\r", "\n" );
		$val = array("&lt;", "&gt;", "&amp;","","");
		$repl = str_replace($key,$val,$line);
		$this->out->append($repl);
	}

}

function markdown($s)
{
	$m=new Markdown();
	return $m->convert($s);
}
